/*
Arduino Poker Timer
Written by Pat James

Inspired and Enabled By:
  Arduino http://arduino.cc
  SparkFun Electronics VoiceBox Shield http://www.sparkfun.com/commerce/product_info.php?products_id=9624
  Practical Arduino http://www.practicalarduino.com/projects/speech-synthesizer
  Speakjet Yahoo! Group http://tech.groups.yahoo.com/group/speakjet/

Features:
  20 minute poker tournament round timer
  Pause/Resume clock
  Add/Subtract 1 minute from clock
  Display minutes remaining in binary using 5 LEDs
  Announce time at 5 minute intervals and final countdown
  Announce minutes remaining on demand
  Speak random phrases on demand, some of which pertain to poker
*/

//Soft serial library used to send serial commands on pin 2 instead of regular serial pin.
#include <SoftwareSerial.h>

// used to include on Serial.print debug statements
#define DEBUG 1

//Arduino Pin Numbers
#define resetPin  3
#define speakingPin  4
#define bufferHalfFullPin 11
#define txPin  2

// LED Output Pins
#define b10000Pin 9 // binary clock LED for "16" bit
#define b01000Pin 8 // binary clock LED for "8" bit
#define b00100Pin 7 // binary clock LED for "4" bit
#define b00010Pin 6 // binary clock LED for "2" bit
#define b00001Pin 5 // binary clock LED for "1" bit

// Pushbutton pins
// Normally open pushbuttons with resistors wired up
#define pauseStartPin 19 // button to pause or start timer.  I am using analog pins 0 - 4 for digital button inputs
#define subtractMinutePin 17 // button to announce current minutes remaining

// Normally closed pushbuttons
#define addMinutePin 16 // button to utter a random phrase
#define announcePin 15 // button to add one minute to timer
#define randomPhrasePin 14 // button to subtract one minute from timer

/* Daisy Bell allophones & SpeakJet aliases obtained from SpeakJet Yahoo! Group
The frequencies of the diatonic musical scale, octaves 1 through
3. (rounded to the nearest integer.. :( */

#define C1 33
#define Cs1 35 // C#1
#define D1 37
#define Ds1 39 // D#1
#define E1 41
#define F1 44
#define Fs1 46 // F#1
#define G1 49
#define Gs1 52 // G#1
#define A1 55
#define As1 58 // A#1
#define B1 62

#define C2 65
#define Cs2 69
#define D2 73
#define Ds2 78
#define E2 82
#define F2 87
#define Fs2 93
#define G2 98
#define Gs2 104
#define A2 110
#define As2 117
#define B2 124

#define C3 131
#define Cs3 139
#define D3 147
#define Ds3 156
#define E3 165
#define F3 175
#define Fs3 185
#define G3 196
#define Gs3 208
#define A3 220
#define As3 233
#define B3 247

// SpeakJet speech element codes

#define IY 128
#define IH 129
#define EY 130
#define EH 131
#define AY 132
#define AX 133
#define UX 134
#define OH 135
#define AW 136
#define OW 137
#define UH 138
#define UW 139
#define MM 140
#define NE 141
#define NO 142
#define NGE 143
#define NGO 144
#define LE 145
#define LO 146
#define WW 147
#define RR 148
#define IYRR 149
#define EYRR 150
#define AXRR 151
#define AWRR 152
#define OWRR 153
#define EYIY 154
#define OHIY 155
#define OWIY 156
#define OHIH 157
#define IYEH 158
#define EHLE 159
#define IYUW 160
#define AXUW 161
#define IHWW 162
#define AYWW 163
#define OWWW 164
#define JH 165
#define VV 166
#define ZZ 167
#define Zh 168
#define DH 169
#define BE 170
#define BO 171
#define EB 172
#define OB 173
#define DE 174
#define DO 175
#define ED 176
#define OD 177
#define GE 178
#define GO 179
#define EG 180
#define OG 181
#define CH 182
#define HE 183
#define HO 184
#define WH 185
#define FF 186
#define Se 187
#define SO 188
#define SH 189
#define TH 190
#define TT 191
#define TU 192
#define TS 193
#define KE 194
#define KO 195
#define EK 196
#define OK 197
#define Pe 198
#define PO 199
#define P0 0
#define P1 1
#define P2 2
#define P3 3
#define P4 4
#define P5 5
#define P6 6
#define FST 7
#define SLW 8
#define STR 14
#define RLX 15
#define PIT 22
#define SPD 21
#define VOL 20
#define BND 23
#define RST 31
#define DELAY 30

// Amusing phrases and historically significant homages to speech synthesis
byte daisy[] ={ \
SPD,114,PIT,D3,DE,SPD,22,EYIY,SPD,114,PIT,B2,ZZ,SPD,5,IY,SPD, \
114,2,1,PIT,G2,DE,SPD,22,EYIY,SPD,114,PIT,D2,ZZ,SPD,5,IY,SPD,114,2,1, \
PIT,E2,GE,IH,VV,0,PIT,F2,MM,IY,0,PIT,G2,IYEH,OWRR,0,PIT,E2,AY \
,NO,2,2,PIT,G2,SO,EYRR,0,PIT,D2,DO,SPD,2,IHWW,SPD,114,2,1, \
PIT,A2,SPD,69,OHIH,SPD,114,MM,2,1,PIT,D3,STR,HO,SPD,17,AY,SPD \
,114,FF,2,PIT,B2,KO,RR,SPD,49,EYIY,SPD,114,1,PIT,G2,ZZ,SPD,5,IY,SPD,114,2,1, \
PIT,E2,SPD,104,AW,SPD,114,LO,0,PIT,F2,FF,SPD,120,OW,SPD,117,AXRR, \
SPD,114,0,PIT,G2,TH,UX,0,PIT,A2,LO,UX, \
SLW,VV,2,2,PIT,B2,UX,VV,0,PIT,A2,\
IYEH,SPD,36,IHWW,SPD,114,3,1,PIT,B2,IH,TT,0, \
PIT,C3,WW,OW,NE,TT,PIT,B2,BE,IY,PIT,A2,UX,PIT,D3,Se,TU,SPD,87 \
,OHIH,SPD,114,PIT,B2,LE,IH,SH,PIT,A2,MM,EYRR,PIT,G2,SPD,0,IH,IH,SPD,114,JH,PIT,A2,OHIY, \
PIT,B2,KE,SPD,56,AY,SPD,114,NO,TT,PIT,G2,SPD,69,UX,SPD,114,FF \
,PIT,E2,SPD,75,OWRR,SPD,114,ED,PIT,G2,SPD,69,UX,SPD,114,PIT,E2,KE,EYRR, \
PIT,D2,SPD,44,IH,IH,SPD,114,JH,BO,SPD,80,UX,SPD,114,TT, \
PIT,G2,IYEH,IHWW,LE,PIT,B2,LE,UH,OK,PIT,A2,Se,WW,IY,TT,2,PIT, \
G2,SPD,100, \
UX,PO,SLW,OH,NO, \
PIT,B2,TH,UX,1,PIT,A2,Se,IY,TT,2,PIT,B2,UX,VV,PIT,C3,EYIY, \
PIT,D3,BE,OHIH,PIT,B2,SO,IH,PIT,G2,KO,LO,PIT,A2,BE,IH,SPD,82, \
EHLE,SPD,114,TT,PIT,D2,FF,OWRR,PIT,G2,TU,UW,SPD,1,UW,UW,SPD,114,3};
byte hutch[] = {8,160,2,152,002,154,128,002,8,184,134,8,182 };
byte notAboutCards[] = {SLW,PO,OW,KE,AXRR,P2,SLW ,IH ,ZZ,P2,NE ,OH ,TT,P2,EYIY ,IY,P2,SLW ,GE ,EYIY ,MM ,P2,UX ,OB ,AYWW ,TT  ,P2,KO ,SLW ,AW ,RR,ED,ZZ, \
  P3,SLW,PO,OW,KE,AXRR,P2,SLW ,IH ,ZZ,P2,EYIY ,IY,P2,SLW ,GE ,EYIY ,MM ,P2,UX ,OB ,AYWW ,TT  ,P2,STR,SLW,Pe ,IY, SLW,Pe ,AX,AX,SLW ,LO};
byte chickenFightLikeaRobot[] = {SPD ,115 ,BND ,4 ,CH ,IH ,KE ,EH ,NE,P3, SLW ,FF ,FST ,OHIY ,P4 ,TT,P2, LE ,FST ,OH ,FST ,OHIY ,EK,P2,EYIY ,IY, P2,RR ,FST ,OW ,FST ,OWWW  ,BO ,OH ,TT ,RST};
byte wildCards[] = {WW,OHIH,EHLE,ED,P2,KE,AWRR,ED,TS,P2,SLW ,SO ,UX ,PO,OWRR,TT,P2,EYIY ,IY,P2,WW ,SLW ,IY ,EK,P2,HE ,SLW,AY ,SLW ,NE ,OD,P2,P2};
byte aces[] = {SLW,EYIY ,SLW,SO,EH,SLW,SO,P2,AWRR,P2,LE ,AWRR ,JH,AXRR,P2,DH ,SLW ,AY ,SLW ,NO,P2,LE ,OHIY ,FST ,IY,FF,P2,SLW ,AY ,SLW ,NE ,OD,P2,SLW ,GO ,FST ,RR ,EYIY ,TT,AXRR,P2,DH ,SLW ,AY ,SLW ,NO,P2,MM ,FST ,AYWW ,NE ,TT ,EY ,NE,ZZ};
byte ironMan[] = {RST ,BND ,2 ,SPD ,72 ,A1 ,OHIH ,IH ,DELAY ,23 ,SPD ,68 ,C2 ,AY ,FST ,AY ,MM ,DELAY ,25 ,FST ,OHIH ,IH ,FST ,D2 ,AX ,SPD ,90 ,RR ,FST ,NO ,FST ,STR ,MM ,AY ,FST ,AY ,SLW ,NE ,RST };
byte stopTheHumanoid[] = {SE ,TT ,SLW ,OH ,PO,P2,SLW ,DH ,SLW ,UX,P2,HO ,FST ,IYUW ,MM ,UX ,FST ,NO ,FST ,OW ,FST ,OWIY ,ED,P2,SE ,TT ,SLW ,OH ,PO,P2,SLW ,DH ,SLW ,UX,P2,IH ,NE ,TT ,RR ,UW ,FST ,DE ,FST ,AX ,FST ,AXRR};

byte* randomPhrases[7]={notAboutCards,aces,daisy,wildCards,chickenFightLikeaRobot, ironMan,stopTheHumanoid};
size_t randomPhraseSizes[7] = {sizeof(notAboutCards),sizeof(aces),sizeof(daisy ),sizeof(wildCards),sizeof(chickenFightLikeaRobot),sizeof(ironMan),sizeof(stopTheHumanoid)};



// timer anouncement phrases

byte readyforaction[] = {RST,SLW,PO,OW,KE,AXRR,P2,SLW, TT, OHIH ,SLW,MM,AXRR,P2,SLW,RR,EY,DE,IY,P2,FF,FST,OW,OWRR,P2,AY,KE,TT,KE,SH,AX,NE \
,P2,SH ,SLW ,AY ,SLW ,LO,P2,WW ,SLW ,IY,P2,PO ,FST ,LE ,EH ,EYIY,P2,PIT,F2,EYIY ,IY,P2,PIT,G2,SLW ,GE ,EYIY ,MM,RST};
byte shuffleup[] = {22,100,8, 189, 7, 134, 8,186, 159,2,8,134,8,199,2,8,132,8,141,8,177,2,8,14,174,128,8,146};

// allophones for announcing current minutes remaining on timer
byte between[] = {BE,IY,TT,WW,SLW,IY,NE,P2};
byte andPhrase[] = {SLW,AY,SLW,NE,OD,P2};
byte minutesremaining[] = {140,7,129,141,129,191,2,148,128,7,140,154,141,128,8, 143};

// allophones to put in an array indexed by minute number
byte twenty[] = {8,7,191,7,147,131,141,007,191,128,P2};
byte nineteen[] = {NE ,FST ,RLX ,OHIY ,NE ,TT ,IY ,SLW ,NE,P2};
byte eighteen[] = {EYIY ,P4 ,TT ,IY ,NE,P2 };
byte seventeen[] = {SLW ,SE ,FST ,EH ,FST ,VV ,FST ,EH ,NE ,FST ,TT ,IY ,NE ,P2  };
byte sixteen[] = {SLW ,SE ,FST ,IH ,STR ,KE ,SE ,FST ,TT ,IY ,NE,P2  };
byte fifteen[] = {186, 129, 186, 191, 128, 141,P2};
byte fourteen[] = {FF ,FST ,OWRR ,FST ,TT ,IY ,NE,P2  };
byte thirteen[] = {SLW ,TH ,FST ,AXRR ,TT ,IY ,NE,P2 };
byte twelve[] = {SLW ,TT ,FST ,WW ,EH ,LE ,VV,P2};
byte eleven[] = {FST ,IH ,LE ,EH ,VV ,FST ,EH ,NE ,P2};
byte ten[] = {191, 131, 131, 141,P2};
byte nine[] = {NE ,STR ,OHIH ,NE,P2};
byte eight[] = {EYIY ,P4 ,TT,P2};
byte seven[] = {SLW ,SE ,FST ,EH ,VV ,EH ,NE,P2};
byte six[] = {SLW ,SE ,IH ,STR ,KE ,FST ,SE,P2};
byte five[] = {186, 157, 166,P2};
byte four[] = {FF ,FST ,OW ,OWRR,P2};
byte three[] = {SLW ,TH ,RR ,SLW ,IY,P2};
byte two[] = {SLW ,TT ,IHWW,P2};
byte one[] = {WW ,STR ,OH ,SLW ,NE,P2};
byte zero[] = {ZZ ,FST ,IY ,FST ,IYRR ,OWWW,P2};

byte* numbers[21]={zero,one,two,three,four,five,six,seven,eight,nine,ten,eleven,twelve,thirteen,fourteen,fifteen,sixteen,seventeen,eighteen,nineteen,twenty};
size_t numberSizes[21] = {sizeof( zero),sizeof(one),sizeof(two),sizeof(three),sizeof(four),sizeof(five),sizeof(six),sizeof(seven),sizeof(eight),sizeof(nine),sizeof(ten),sizeof(eleven),sizeof(twelve),sizeof(thirteen),sizeof(fourteen),sizeof(fifteen),sizeof(sixteen),sizeof(seventeen),sizeof(eighteen),sizeof(nineteen),sizeof(twenty)};

byte countdown[] = {29,5};
byte blindsareup[] = {8, 169, 8, 128, 2,171,145,155,141,177,187,187,2,152,2,8,134,199};
byte blindluck[] = {29,7,1,8, 169, 8, 128, 2,171,145,155,141,177,187,187,2,152,2,8,134,199, 2, 2, 8, 179, 138, 138, 177 ,2,146, 8, 134, 8, 194,2,1,14,199,7, 145, 8,131, 8,158,8,187,187,191};

byte timerpaused[] = {SLW ,TT ,OHIH ,SLW ,MM,AXRR,P2,PO ,SLW ,OH ,OH,ZZ,ED};
byte timerresumed[] = {SLW ,TT ,OHIH ,SLW ,MM,AXRR,P2,RR ,FST ,IY,ZZ,UW,MM, ED};

byte twiki[] = {29,7};
byte r2d2[] = {29,4};

// global timer control variables
int roundLengthMinutes = 20;
int minutesRemaining = roundLengthMinutes;
int secondsRemaining = roundLengthMinutes * 60;
int roundLengthSeconds = roundLengthMinutes * 60;
boolean timerStarted = false;
boolean timerPaused = false;
boolean timerPausedAndLightsOut = false;
int randomPhraseNumber = 0;
long startMillis = 0;
long pauseStartMillis = 0;
long pausedMillis = 0;
boolean fifteenMinuteWarningDone = false;
boolean tenMinuteWarningDone = false;
boolean fiveMinuteWarningDone = false;
boolean oneMinuteWarningDone = false;
boolean tenSecondWarningDone = false;

// push buttom state variables
int pauseStartState = LOW;
int addMinuteState = LOW;
int subtractMinuteState = LOW;
int randomPhraseState = LOW;
int announceState = LOW;

//Create a SoftSerial Object for communicating to Speakjet
SoftwareSerial speakjet = SoftwareSerial(0, txPin);

void setup()  
{
  #ifdef DEBUG
  Serial.begin(9600); // for debugging
  #endif
  
  //Configure the pins for the SpeakJet module
  pinMode(bufferHalfFullPin, INPUT); // for throttling SpeakJet input when 64 byte buffer is half full
  pinMode(txPin, OUTPUT); // for sending commands to SpeakJet
  pinMode(speakingPin, INPUT); // so we don't interrupt SpeakJet when it is already speaking

  // Configure LED pins
  pinMode(b10000Pin, OUTPUT);
  pinMode(b01000Pin, OUTPUT);
  pinMode(b00100Pin, OUTPUT);
  pinMode(b00010Pin, OUTPUT);
  pinMode(b00001Pin, OUTPUT);
  
  /* Configure button pins
     Note my buttons are a mixed bag...some normally closed
     some normally open, some with external pull-down resistors, some
     with internal pull-up resistors.  If I did it over, I would
     use the internally pull-up resistors for all of them */
  pinMode(subtractMinutePin, INPUT);           // set pin to input
  digitalWrite(14, HIGH);       // turn on internal pullup resistor
  pinMode(addMinutePin, INPUT);
  digitalWrite(15, HIGH);       // turn on internal pullup resistor
  digitalWrite(16, HIGH);       // turn on internal pullup resistor
  pinMode(pauseStartPin, INPUT);

  pinMode(randomPhrasePin, INPUT);
  pinMode(announcePin, INPUT);

  //Set up a serial port to talk from Arduino to the SpeakJet module on pin 3.
  speakjet.begin(9600);
  delay(500);

  // Announce that we are Ready to Go!
  speak(readyforaction, sizeof(readyforaction)/sizeof(byte));
}

/* 
Button Press Actions

Start/Pause/Resume:  
  if no timer running, start timer
  if timer running, pause timer & flash all lit LEDs
  if timer paused, resume timer and light LEDs for the current time
  
+1m:
  add one minute to timer
  adjust LEDs accordingly
  
-1m:
  subtract one minute from timer
  adjust LEDs accordingly

Announce:
  single press: speak time remaining, rounded to nearest minute
  double press: speak current battery level (not implemented yet)
  
???:
  speak a random phrase for general amusement  

*/
void loop()
{ 
  delay(250);
  Serial.begin(9600);
  
  // Read push button state
  addMinuteState = digitalRead(addMinutePin);
  subtractMinuteState = digitalRead(subtractMinutePin);
  announceState = digitalRead(announcePin);
  randomPhraseState = digitalRead(randomPhrasePin);
  pauseStartState = digitalRead(pauseStartPin);

  if (addMinuteState == HIGH) {
    #ifdef DEBUG
    Serial.println("addMinuteState HIGH");
    #endif
    roundLengthSeconds += 60;
    minutesRemaining += 1;
    updateBinaryClockLEDs();
    announceMinutesRemaining();
  }
  
  if (subtractMinuteState == HIGH) {
    #ifdef DEBUG
    Serial.println("subtractMinuteState HIGH");
    #endif
    roundLengthSeconds -= 60;
    minutesRemaining -= 1;
    updateBinaryClockLEDs();
    announceMinutesRemaining();
  }
  
  if (announceState == HIGH) {
    #ifdef DEBUG
    Serial.println("announceState HIGH");
    #endif
    announceMinutesRemaining();
  }
  
  if (randomPhraseState == HIGH) {
    #ifdef DEBUG
    Serial.println("randomPhraseState HIGH");
    #endif
    utterRandomPhrase();
  }

  if (pauseStartState == HIGH) {     
    #ifdef DEBUG
    Serial.println("pauseStartState HIGH");
    #endif
    if (timerStarted) {
      // clock is running, so treat button press as pause/resume toggle
      if (timerPaused) {
        speak(timerresumed, sizeof(timerresumed)/sizeof(byte));
        timerPaused = false;
        timerPausedAndLightsOut = false;
        updateBinaryClockLEDs();
        pausedMillis = pausedMillis + (millis() - pauseStartMillis); 
        #ifdef DEBUG     
        Serial.print("Resume: ");
        Serial.print(secondsRemaining);
        Serial.println("");  
        #endif
      } else {
        speak(timerpaused, sizeof(timerpaused)/sizeof(byte));
        timerPaused = true;
        pauseStartMillis = millis();
        #ifdef DEBUG
        Serial.print("Pause: ");
        Serial.print(secondsRemaining);
        Serial.println("");  
        #endif
      }
    } else {
      // clock not running yet, so start it
      speak(shuffleup, sizeof(shuffleup)/sizeof(byte));
      startTimer();
      speak(twenty, sizeof(twenty)/sizeof(byte));
      speak(minutesremaining, sizeof(minutesremaining)/sizeof(byte));
      
    }
  }
  updateTimer();
}

void startTimer() {
  timerStarted = 1;
  pausedMillis = 0;
  startMillis = millis();
  minutesRemaining = roundLengthMinutes;
  secondsRemaining = roundLengthMinutes * 60;
  roundLengthSeconds = roundLengthMinutes * 60;
  timerPaused = 0; // probably not needed, but why not be safe?
  timerPausedAndLightsOut = 0;
  startMillis = millis();
  pausedMillis = 0;
  fifteenMinuteWarningDone = false;
  tenMinuteWarningDone = false;
  fiveMinuteWarningDone = false;
  oneMinuteWarningDone = false;
  tenSecondWarningDone = false;
}

// things to do when timer is running
void updateTimer() {
  if (timerStarted && !timerPaused) {
    long thisMillis = millis();
    int elapsedSeconds = (thisMillis - startMillis - pausedMillis) / 1000;
    secondsRemaining = roundLengthSeconds - elapsedSeconds;
    minutesRemaining = secondsRemaining / 60;
    #ifdef DEBUG
    Serial.print("thisMillis: ");
    Serial.print(thisMillis,DEC);
    Serial.print(", paused millis: ");
    Serial.print(pausedMillis,DEC);
    Serial.print(", elapsed sec: ");
    Serial.print(elapsedSeconds,DEC);
    Serial.print(", remaining seconds: ");
    Serial.println(secondsRemaining,DEC);
    #endif
   
    if (secondsRemaining <= 900 && !fifteenMinuteWarningDone && roundLengthMinutes >= 15) {
        speak(twiki, sizeof(twiki)/sizeof(byte));
        speak(fifteen, sizeof(fifteen)/sizeof(byte));
        speak(minutesremaining, sizeof(minutesremaining)/sizeof(byte));
        fifteenMinuteWarningDone = true;
    }

    if (secondsRemaining <= 600 && !tenMinuteWarningDone && roundLengthMinutes >= 10) {
      tenMinuteWarningDone = true;
      speak(twiki, sizeof(twiki)/sizeof(byte));
      speak(ten, sizeof(ten)/sizeof(byte));
      speak(minutesremaining, sizeof(minutesremaining)/sizeof(byte));
    }
    
    if (secondsRemaining <= 300 && !fiveMinuteWarningDone && roundLengthMinutes >= 5) {
      speak(twiki, sizeof(twiki)/sizeof(byte));
      speak(five, sizeof(five)/sizeof(byte));
      speak(minutesremaining, sizeof(minutesremaining)/sizeof(byte));
      fiveMinuteWarningDone = true;
    }

    if (secondsRemaining <= 60 && !oneMinuteWarningDone) {
      speak(twiki, sizeof(twiki)/sizeof(byte));
      speak(one, sizeof(one)/sizeof(byte));
      speak(minutesremaining, sizeof(minutesremaining)/sizeof(byte));
      oneMinuteWarningDone = true;
    }
    
    if (secondsRemaining <= 10 && !tenSecondWarningDone) {
      speak(countdown, sizeof(countdown)/sizeof(byte));  
      tenSecondWarningDone = true;
    }
    
    if (secondsRemaining == 0) {
      speak(blindluck, sizeof(blindluck)/sizeof(byte));
      startTimer();
    }

    updateBinaryClockLEDs();
  }
  
  // while paused, any LEDs currently lit get flashed off/on
  // as a visual reminder that clock is paused
  if (timerPaused) {
    if (timerPausedAndLightsOut) {
        updateBinaryClockLEDs();
    } else {
      digitalWrite(b00001Pin, LOW);
      digitalWrite(b00010Pin, LOW);
      digitalWrite(b00100Pin, LOW);
      digitalWrite(b01000Pin, LOW);
      digitalWrite(b10000Pin, LOW);
    }
    timerPausedAndLightsOut = !timerPausedAndLightsOut;
  }
}

/*
  updateBinaryClockLEDs
    Display minutes remaining on the 5 LED binary "clock"
    Yeah, coulda used a more human-readable visual display,
    but where's the fun in that?  Not quite geeky enough :)
*/
void updateBinaryClockLEDs() {
  int minutesToDisplay = minutesRemaining - 0;
  if (secondsRemaining > 50) {
    // normal binary clock display
    if ((minutesToDisplay & 1) > 0) {
      digitalWrite(b00001Pin, HIGH);
    } else {
      digitalWrite(b00001Pin, LOW);
    }
    if ((minutesToDisplay & 2) > 0) {
      digitalWrite(b00010Pin, HIGH);
    } else {
      digitalWrite(b00010Pin, LOW);
    }
    if ((minutesToDisplay & 4) > 0) {
      digitalWrite(b00100Pin, HIGH);
    } else {
      digitalWrite(b00100Pin, LOW);
    }
    if ((minutesToDisplay & 8) > 0) {
      digitalWrite(b01000Pin, HIGH);
    } else {
      digitalWrite(b01000Pin, LOW);
    }
    if ((minutesToDisplay & 16) > 0) {
      digitalWrite(b10000Pin, HIGH);
    } else {
      digitalWrite(b10000Pin, LOW);
    }
  } else {
    // final 50 seconds, use LEDs as countdown in 10 second intervals
    digitalWrite(b10000Pin, LOW);
    digitalWrite(b01000Pin, LOW);
    digitalWrite(b00100Pin, LOW);
    digitalWrite(b00010Pin, LOW);
    digitalWrite(b00001Pin, LOW);
    if (secondsRemaining >= 10 ) digitalWrite(b10000Pin, HIGH);
    if (secondsRemaining >= 20 ) digitalWrite(b01000Pin, HIGH);
    if (secondsRemaining >= 30 ) digitalWrite(b00100Pin, HIGH);
    if (secondsRemaining >= 40 ) digitalWrite(b00010Pin, HIGH);
    if (secondsRemaining >= 50 ) digitalWrite(b00001Pin, HIGH);
  }
}

void utterRandomPhrase() {
//  Serial.println("Daisy");
//  speak(randomPhrases[0], randomPhraseSizes[0]/sizeof(byte));
  speak(randomPhrases[randomPhraseNumber], randomPhraseSizes[randomPhraseNumber]/sizeof(byte));
  if (randomPhraseNumber == 6) {
   randomPhraseNumber = 0;
  } else {
    randomPhraseNumber +=1;
    
  }
}

void announceMinutesRemaining() {
   int previousMinute = minutesRemaining + 1;
   speak(between, sizeof(between)/sizeof(byte));
   speak(numbers[minutesRemaining], numberSizes[minutesRemaining]/sizeof(byte));
   speak(andPhrase, sizeof(andPhrase)/sizeof(byte));
   speak(numbers[previousMinute], numberSizes[previousMinute]/sizeof(byte));
   speak(minutesremaining, sizeof(minutesremaining)/sizeof(byte));
}

/* 
  speak
    Sends phrase to SpeakJet
    Waits if SpeakJet buffer is half full or if SpeakJet currently speaking 
*/
void speak(byte phrase[], int length) {
  speakjet.print(RST, BYTE); // reset pitch, speed, volume
  int speaking = digitalRead(speakingPin);
  while (speaking == 1) {
      delay(100);  // it is rude to interrupt someone while they are speaking!
      speaking = digitalRead(speakingPin);
  }
  int i=0;
  int bufferHalfFull = 0;
  while (i < length) {
    bufferHalfFull = digitalRead(bufferHalfFullPin);
    while (bufferHalfFull == 1) {
      delay(100);  // wait for buffer to clear before sending more allophones
      bufferHalfFull = digitalRead(bufferHalfFullPin);
    }
    speakjet.print(phrase[i], BYTE); // send allophone to SpeakJet
    i++;
  }
}


